#include "of_str.h"
#include <chrono>
#include <ctime>
#include <iomanip>
#include <sstream>
#include <string>
#include <utf8.h>

namespace ofen {
COfStr::COfStr()
{
}
COfStr::~COfStr()
{
}
ofString COfStr::replace(const ofString& str, const ofString& from, const ofString& to)
{
    if (from.empty()) {
        return str;
    }
    ofString result = str;
    size_t startPos = 0;
    while ((startPos = result.find(from, startPos)) != ofString::npos) {
        result.replace(startPos, from.length(), to);
        startPos += to.length();
    }
    return result;
}
std::vector<ofString> COfStr::split(const ofString& input, const ofString& delimiter)
{
    std::vector<ofString> result;
    size_t pos = 0, prev = 0;
    while ((pos = input.find(delimiter, prev)) != ofString::npos) {
        result.push_back(input.substr(prev, pos - prev));
        prev = pos + delimiter.size();   // Move past the delimiter
    }
    result.push_back(input.substr(prev));
    return result;
}
ofString COfStr::trim(const ofString& input)
{
    size_t start = input.find_first_not_of(ofT(" \t\n\r\f\v"));
    if (start == ofString::npos) {
        return ofT("");
    }
    size_t end = input.find_last_not_of(ofT(" \t\n\r\f\v"));
    return input.substr(start, end - start + 1);
}
ofString COfStr::get_ofile_name(const ofString& name)
{
    auto now = std::chrono::system_clock::now();
    std::time_t now_c = std::chrono::system_clock::to_time_t(now);
    std::tm local_tm = *std::localtime(&now_c);

    std::ostringstream oss;
    oss << std::put_time(&local_tm, "%Y%m%d%H%M%S");
    ofString timestamp;
#ifdef UNICODE_OFSTR
    std::string temp = oss.str();
    timestamp.assign(temp.begin(), temp.end());
#else
    timestamp = oss.str();
#endif
    size_t dot_pos = name.find_last_of(ofT('.'));
    if (dot_pos == ofString::npos) {
        return name + ofT("_") + timestamp;
    }
    ofString base_name = name.substr(0, dot_pos);
    ofString extension = name.substr(dot_pos);
    return base_name + ofT("_") + timestamp + extension;
}
std::string COfStr::u8_convert(const std::wstring& source)
{
    std::string utf8_result;
    try {
        utf8::utf16to8(source.begin(), source.end(), std::back_inserter(utf8_result));
    } catch (const std::exception& e) {
        return "";
    }
    return utf8_result;
}
std::wstring COfStr::u8_convert(const std::string& source)
{
    std::wstring wide_result;
    try {
        utf8::utf8to16(source.begin(), source.end(), std::back_inserter(wide_result));
    } catch (const std::exception& e) {
        return L"";
    }
    return wide_result;
}
}   // namespace ofen
